/*

  Copyright (c) 2009-2018, Thanos Theo. All rights reserved.
  Released Under a Simplified BSD (FreeBSD) License
  for academic, personal & non-commercial use.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

  The views and conclusions contained in the software and documentation are those
  of the author and should not be interpreted as representing official policies,
  either expressed or implied, of the FreeBSD Project.

  A Commercial License is also available for commercial use with
  special restrictions and obligations at a one-off fee. See links at:
  1. http://www.dotredconsultancy.com/openglrenderingenginetoolrelease.php
  2. http://www.dotredconsultancy.com/openglrenderingenginetoolsourcecodelicence.php
  Please contact Thanos Theo (thanos.theo@dotredconsultancy.com) for more information.

*/

/**
*
*  Apply Cylinder Model GLSL code.
*  Using core functions from CommonFunctions & SurfaceLightingModels GLSL libraries.
*
*  Relevant variables controlling the above: useActorColor, actorColor & lightingModel
*
* @author Thanos Theo, 2009-2018
* @version 14.0.0.0
*/

uniform float radiusFactor = 1.0;
uniform bool useActorColor = false;
uniform vec3 actorColor = vec3(1.0);
uniform bool useRandomlyGeneratedCylinderVariables = true;
uniform float cylinderRadius = 1.0 / 250.0;

out float cylinderRadiusIn;
out float cylinderHeightIn;
out vec3  cylinderDirectionIn;
out vec4  cylinderColorIn;

float rand1(in vec2 seed);
vec3 cosPowHemiRandom(in vec2 seed, float k);

void main()
{
  if (useRandomlyGeneratedCylinderVariables)
  {
    cylinderRadiusIn = radiusFactor * gl_Vertex.w / 4.0;
    cylinderHeightIn = rand1(vec2(cylinderRadiusIn)) / 2.0;
    cylinderDirectionIn = normalize(cosPowHemiRandom(gl_Vertex.xy, gl_Vertex.z));
  }
  else
  {
    cylinderRadiusIn = radiusFactor * cylinderRadius;
    cylinderHeightIn = gl_Vertex.w;
    cylinderDirectionIn = normalize(gl_Normal);
  }
  cylinderColorIn = (useActorColor) ? vec4(actorColor, 1.0) : gl_Color;
  gl_Position     = vec4(gl_Vertex.xyz, 1.0);
  gl_ClipVertex   = vec4(gl_Vertex.xyz, 1.0);
}