/*

  Copyright (c) 2009-2018, Thanos Theo. All rights reserved.
  Released Under a Simplified BSD (FreeBSD) License
  for academic, personal & non-commercial use.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

  The views and conclusions contained in the software and documentation are those
  of the author and should not be interpreted as representing official policies,
  either expressed or implied, of the FreeBSD Project.

  A Commercial License is also available for commercial use with
  special restrictions and obligations at a one-off fee. See links at:
  1. http://www.dotredconsultancy.com/openglrenderingenginetoolrelease.php
  2. http://www.dotredconsultancy.com/openglrenderingenginetoolsourcecodelicence.php
  Please contact Thanos Theo (thanos.theo@dotredconsultancy.com) for more information.

*/

/**
*
*  Apply Point Sphere Model GLSL code.
*  Using core functions from CommonFunctions & SurfaceLightingModels GLSL libraries.
*
*  Relevant variables controlling the above: useActorColor, actorColor, vpSizeX-Y, vpOriginX-Y, variablePointSphereRadius & pointSphereDiameterWithTiling
*
* @author Thanos Theo, 2009-2018
* @version 14.0.0.0
*/

uniform bool useActorColor = false;
uniform vec3 actorColor = vec3(1.0);
uniform vec4 vpSizeAndOrigin = vec4(0.5);
uniform bool variablePointSphereRadius = false;
uniform float pointSphereDiameterWithTiling = 1.0;
uniform bool pointAntialias = false;

VS_VARYING float sphereRadius;
VS_VARYING float sphereCircularFactor;
VS_VARYING vec4 sphereColor;
VS_VARYING vec2 sphereClipPosition;
VS_VARYING vec3 cameraSpherePosition;

const float MAX_POINT_SIZE_PIXEL_RADIUS = 64.0;

float getSphericalDistanceFromCenter(in float radius, in vec3 center);

void main()
{
  if (variablePointSphereRadius)
  {
    // for constant point sprite size while zooming
    // gl_Position = gl_ModelViewProjectionMatrix * vec4(gl_Vertex.xyz, 1.0);
    // float sphereDiameter = 20.0 * MAX_POINT_SIZE_PIXEL_RADIUS * gl_Vertex.w * pointSphereDiameterWithTiling / gl_Position.z;
    float sphereDiameter = MAX_POINT_SIZE_PIXEL_RADIUS * gl_Vertex.w * pointSphereDiameterWithTiling;
    gl_PointSize = sphereDiameter;
    sphereRadius = sphereDiameter / 2.0;
    if (pointAntialias)
    {
      sphereCircularFactor = getSphericalDistanceFromCenter(sphereRadius, gl_Vertex.xyz);
    }
  }
  else
  {
    gl_PointSize = pointSphereDiameterWithTiling;
  }

  vec4 vertex = gl_ModelViewMatrix * vec4(gl_Vertex.xyz, 1.0);
  cameraSpherePosition = vertex.xyz;
  sphereColor = (useActorColor) ? vec4(actorColor, 1.0) : gl_Color;
  gl_Position = gl_ProjectionMatrix * vertex;
  gl_ClipVertex = vertex;

  // compute the sphere position in window coordinates
  sphereClipPosition = gl_Position.xy / gl_Position.w;
  sphereClipPosition = sphereClipPosition * vec2(0.5) + vec2(0.5);
  sphereClipPosition = sphereClipPosition * vpSizeAndOrigin.xy + vpSizeAndOrigin.zw;
}